# ====================================== #
#      Animation of the Pendulum         #
#      Summer School and Workshop on     #
#  Modeling and Numerics in Epidemiology #
#                                        #
#      Cali, Colombia. 09.12.2024        #
#                 DAAD                   #
#========================================#

import numpy as np
import matplotlib.pyplot as plt
from matplotlib.animation import FuncAnimation

# Constants
L = 1.0  # Length of pendulum (m)

# Use the finest solution (t3 and phi3 from previous exercise)
t3 = np.linspace(0, 10, 400)  # Finest time step
phi3 = np.loadtxt("pendulum_data.dat")

# Calculate pendulum positions
x = L * np.sin(phi3)  # Horizontal position of the pendulum
y = -L * np.cos(phi3)  # Vertical position of the pendulum

# Create the figure and axis for the animation
fig, ax = plt.subplots()
ax.set_aspect('equal', adjustable='box')
ax.set_xlim(-1.2, 1.2)
ax.set_ylim(-1.2, 1.2)

# Pendulum components
line, = ax.plot([], [], 'o-', lw=2)  # Pendulum rod and bob
trail, = ax.plot([], [], 'r-', alpha=0.5, lw=1)  # Trajectory trail of the bob

# Initialize the plot elements
def init():
    line.set_data([], [])
    trail.set_data([], [])
    return line, trail

# Update function for animation
def update(frame):
    # Current position of the pendulum
    x_pos = x[frame]
    y_pos = y[frame]

    # Update the pendulum rod and bob
    line.set_data([0, x_pos], [0, y_pos])
    
    # Update the trajectory trail
    trail.set_data(x[:frame], y[:frame])
    
    return line, trail

# Create the animation
ani = FuncAnimation(fig, update, frames=len(t3), init_func=init, blit=True, interval=10)

# Display the animation
plt.show()
